
/* Arduino code for translating potentiometer or computer motor commands into pulses that 
 * are sent to the Hybrid Servo Drive system
 * 
 * Some tuning may be required 
 * e.g. tweaking responsiveness to avoid the catch-up thing the hybrid
 * driver does if it can't accelerate as quickly as it is told to.
 */


//---------------variables-----------------

//pins
const int directionPin = 3;         //pin taking the direction input for motor driver
const int pulsePin = 2;             //pin taking the pulse input for motor driver
const int enablePin = 5;            //pin that switches motor on and off (if set to LOW, motor is enabled)
const int potPin = A1;              //pin that potentiometer is attached to
const int potSwitchPin = 6;         //pin that toggle switch is attached to, used to switch between potentiometer and default input
const int externalLedPin = 7;       //pin that controls led on the outside of the control box


//hybrid system
long stepsPerRevolution = 1600;     //how many steps per revolution for the motor? (check switches on "PA Setting" on hybrid servo motor driver)
float responsiveness = 0.01;        //weight given to the goal speed value when working out where to set speed value (between 0 and 1)                <---*use this instead. Simpler and faster
float oneMinusResponsiveness = 1.0 - responsiveness;

//input
int defaultInputType = 1;           //0 = keyboard
                                    //1 = unity program  (which could be using mouse, VR controller, or whatever)
                                    //2 = potentiometer
                                    //NB best to make default either 0 or 1 as there is a toggle switch on the control box that switches between default and potentiometer
int inputType = defaultInputType;   //set to default for now
char kbInput;                       //place holder for input from keyboard
String unityCommand;                //place holder for commands sent (from unity) via serial
int potPosition = 0;                //position of potentiometer
int oldPotSwitchValue;              //previous position of potentiometer switch last time it was flipped


//other control variables
long speedIncrementsDeg = 720;      //divide the speed range (0 - 360 deg/s) into how many increments? The bigger the number the finer the speed changes                                   
long speedValue = 0;                //speed value (in 360/speedIncrementsDeg deg/s)   
long oldSpeedValue = 0;             //speed at last time time step
long goalSpeedValue = 0;            //goal speed value (based on input from user)                                 
unsigned long halfPulseLength;      //time delay between pulses in microseconds x 0.5 (controlled by speed)
unsigned long currentTime;          //curent time for purpose of controlling pulse width
unsigned long lastStepTime;         //time at which the last step occured
bool stepUp = false;                //are we stepping the signal up (or down)? For making square wave signal.
bool oldSpeedPositive = true;       //is the sign of the old speedValue positive? For deciding if need to change direction of chair.


//ready to move?
bool letsMove = false;              //not yet ready for motor to operate



//---------set up starting conditions-------
void setup() 
{

 //set output pins (to motor driver)
 pinMode(directionPin, OUTPUT);
 pinMode(pulsePin, OUTPUT);
 pinMode(enablePin, OUTPUT);

 //set input pins
 pinMode(potSwitchPin,INPUT_PULLUP);          //INPUT_PULLUP puts 5V on the pin so when switch closed the pin goes to 0V (as other end attached to ground) 

 //turn everything off for now
 digitalWrite(directionPin,LOW);
 digitalWrite(pulsePin,LOW);
 digitalWrite(enablePin,HIGH);                //motor is off when enable is HIGH

 //check potentiometer switch
 oldPotSwitchValue = digitalRead(potSwitchPin);

 //set time between pulses to a large number for now
 halfPulseLength = 100000000;

 //get serial port going
 Serial.begin(9600); 
 Serial.setTimeout(100);  //just to make sure reading a signal doesn't take too long if something goes wrong

 //get step timer going
 lastStepTime = micros();

 //allow motion under certain conditions
 if(inputType == 1 | inputType == 2)
 {
  //get motion happening
  letsMove = true;
  digitalWrite(enablePin,LOW);
 }

}



void loop() 
{  
  //---check position of potentiometer switch---
  int potSwitchValue = digitalRead(potSwitchPin);
  if(potSwitchValue != oldPotSwitchValue)             //if switch has just been flipped...
  {   
    if(potSwitchValue == LOW)                         //and if flipped up...
     {
      digitalWrite(externalLedPin,HIGH);              //turn on LED...
      inputType = 2;                                  //and switch to using potentiometer
     }
     else
     {
      digitalWrite(externalLedPin,LOW);               //otherwise turn LED off
      inputType = defaultInputType;                   //and go back to default input type
     }
     oldPotSwitchValue = potSwitchValue;
  }
 

  //===listen for chair speed commands===
  
  if(inputType == 0)                                      //----Keyboard input----
  {
    if(Serial.available()>0)                              //if a key has been pressed since last checked
    {
      kbInput = Serial.read();                            //read the key
      Serial.println(kbInput);                            //send back to computer (to print or whatever)
    
      if(kbInput == 's')
      {
        letsMove = !letsMove;                             //toggle motor commands on and off
        if(letsMove){digitalWrite(enablePin,LOW);}        //turn motor driver on if motor commands turned on
        else{digitalWrite(enablePin,HIGH);}
        speedValue = 0;                                   //reset speed to 0;
        oldSpeedValue = 0;
        goalSpeedValue = 0;
      }
      
      if(kbInput == 'd')
      {
        goalSpeedValue += 10;                             //increase speed value
      }
      
      if(kbInput == 'a')
      {
        goalSpeedValue -= 10;                             //decrease speed value
      }

    }
    
  }


  if(inputType == 1)                                      //----Unity input----
  {
    if (Serial.available()>3)                             //tbmotion_speed unity code sends info as string of 4 digits e.g. "0512" (meaning speed = 0. First message sent when trial starts
    {
      
      //read serial buffer for commands from unity        //reading the string has to be done in this very specific way otherwise issues occur
      unityCommand = "";                                  //clear old unityCommand value
      int ii = 0;
      while(ii < 4)
      {
        char cc = Serial.read();                          //read one character at a time
        if(isDigit(cc))                                   //if it is a digit (this is to rule out new line characters and stuff)
        {
          unityCommand += cc;                             //then add it to the unityCommand string
          ii++;
        }
      }

      goalSpeedValue = map(unityCommand.toInt(), 0, 1023, -speedIncrementsDeg, speedIncrementsDeg);      //map 0-1023 input space to speedValue space

      //clear the serial buffer ready for next message
      while(Serial.available()>0)
      {
        Serial.read();
      } 

      Serial.println(unityCommand);                       //message needs to be sent back to unity to let it know arduino is ready for next command
    }
  }



  if(inputType == 2)                                      //----Potentiometer input----
  {
    potPosition = analogRead(potPin);                                                         //read potentiometer
    goalSpeedValue = map(potPosition, 0, 1023, -speedIncrementsDeg, speedIncrementsDeg);      //map analogue input space to speedValue space  
  }



  //===send chair movement commands to chair===

  if(letsMove)    //if we are in move mode, then check if its time to send a pulse
  {

    //put a damper on speed changes to make sure they don't exceed capabilities of motor (by using weighted average of old speed and goal speed)
    speedValue = oneMinusResponsiveness*oldSpeedValue + responsiveness*goalSpeedValue;
    

    
    //set direction of motor based on sign of speedValue
    bool speedPositive = (speedValue >=0);                    //check if sign of speedValue positive
    if(speedPositive != oldSpeedPositive)                     //if there has been a change in the sign of the speedValue (it takes more code but speeds up loops to only set direction pin if need to)
    {
      if(speedValue >= 0) {digitalWrite(directionPin,LOW);}   //...then change direction of chair to match
      else{digitalWrite(directionPin,HIGH);}
      oldSpeedPositive = speedPositive;
    }
    

    
    //work out appropriate time delay between pulses based on magnitude of speedValue
    if(speedValue == 0)
    {
      halfPulseLength = 10000000;             //some big number to ensure no pulses any time soon
    }
    else
    {
      halfPulseLength = round( 1000000 * speedIncrementsDeg / (abs(speedValue) * stepsPerRevolution) );
    }


    //check if it is time to turn on (or turn off) a pulse
    currentTime = micros();
    if(currentTime - lastStepTime > halfPulseLength)
    {
      stepUp = !stepUp;
      if(stepUp){digitalWrite(pulsePin,HIGH);}
      else{digitalWrite(pulsePin,LOW);}
      lastStepTime = currentTime;
    }
     
  }

  oldSpeedValue = speedValue;


}
